#!/usr/bin/env python
# _*_coding:utf-8_*_

import sys, platform, os, re
import argparse
import numpy as np

pPath = os.path.split(os.path.realpath(__file__))[0]
sys.path.append(pPath)
father_path = os.path.abspath(
    os.path.dirname(pPath) + os.path.sep + ".") + r'\pubscripts' if platform.system() == 'Windows' else os.path.abspath(
    os.path.dirname(pPath) + os.path.sep + ".") + r'/pubscripts'
sys.path.append(father_path)
import read_fasta_sequences
import save_file
import check_sequences


def Moran(fastas,
          props=['CIDH920105', 'BHAR880101', 'CHAM820101', 'CHAM820102', 'CHOC760101', 'BIGC670101', 'CHAM810101', 'DAYM780201','ARGP820102', 'CHOC760102', 'CRAJ730101', 'CHAM820101', 'CHAM830101', 'HUTJ700101', 'ISOY800102'], nlag=16, **kw):
    if check_sequences.get_min_sequence_length_1(fastas) < nlag + 1:
        print('Error: all the sequence length should be larger than the nlag+1: ' + str(nlag + 1) + '\n\n')
        return 0

    AA = 'ARNDCQEGHILKMFPSTWYV'
    fileAAidx = re.sub('descproteins$', '', os.path.split(os.path.realpath(__file__))[
        0]) + r'\data\AAidx.txt' if platform.system() == 'Windows' else sys.path[0] + '/data/AAidx.txt'

    with open(fileAAidx) as f:
        records = f.readlines()[1:]
    myDict = {}
    for i in records:
        array = i.rstrip().split('\t')
        myDict[array[0]] = array[1:]

    AAidx = []
    AAidxName = []
    for i in props:
        if i in myDict:
            AAidx.append(myDict[i])
            AAidxName.append(i)
        else:
            print('"' + i + '" properties not exist.')
            return None

    AAidx1 = np.array([float(j) for i in AAidx for j in i])
    AAidx = AAidx1.reshape((len(AAidx), 20))

    propMean = np.mean(AAidx, axis=1)
    propStd = np.std(AAidx, axis=1)

    for i in range(len(AAidx)):
        for j in range(len(AAidx[i])):
            AAidx[i][j] = (AAidx[i][j] - propMean[i]) / propStd[i]

    index = {}
    for i in range(len(AA)):
        index[AA[i]] = i

    encodings = []
    header = ['#', 'label']
    for p in props:
        for n in range(1, nlag + 1):
            header.append(p + '.lag' + str(n))
    encodings.append(header)

    for i in fastas:
        name, sequence, label = i[0], re.sub('-', '', i[1]), i[2]
        code = [name, label]
        N = len(sequence)
        for prop in range(len(props)):
            xmean = sum([AAidx[prop][index[aa]] for aa in sequence]) / N
            for n in range(1, nlag + 1):
                if len(sequence) > nlag:
                    # if key is '-', then the value is 0
                    fenzi = sum([(AAidx[prop][index.get(sequence[j], 0)] - xmean) * (
                    AAidx[prop][index.get(sequence[j + n], 0)] - xmean) for j in range(len(sequence) - n)]) / (N - n)
                    fenmu = sum(
                        [(AAidx[prop][index.get(sequence[j], 0)] - xmean) ** 2 for j in range(len(sequence))]) / N
                    rn = fenzi / fenmu
                else:
                    rn = 'NA'
                code.append(rn)
        encodings.append(code)
    return encodings


if __name__ == '__main__':
    parser = argparse.ArgumentParser(usage="it's usage tip.",
                                     description="Moran descriptor")
    parser.add_argument("--file", required=True, help="input fasta file")
    parser.add_argument("--props", help="input the property names, the names were separated by the symbol ':'")
    parser.add_argument("--nlag", help="set the nlag value, default is 30")
    parser.add_argument("--format", choices=['csv', 'tsv', 'svm', 'weka'], default='svm', help="the encoding type")
    parser.add_argument("--out", help="the generated descriptor file")
    args = parser.parse_args()

    fastas = read_fasta_sequences.read_protein_sequences(args.file)
    props = args.props.split(':') if args.props != None else ['CIDH920105', 'BHAR880101', 'CHAM820101', 'CHAM820102', 'CHOC760101', 'BIGC670101', 'CHAM810101', 'DAYM780201','ARGP820102', 'CHOC760102', 'CRAJ730101', 'CHAM820101', 'CHAM830101', 'HUTJ700101', 'ISOY800102']
    nlag = int(args.nlag) if args.nlag != None else 30
    output = args.out if args.out != None else 'encoding.txt'
    
    encodings = Moran(fastas, props, nlag)
    print(encodings)
    save_file.save_file(encodings, args.format, output)
